package javax.management.openmbean;

import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;

public class TabularType extends OpenType<TabularData>
{
    static final long serialVersionUID = 6554071860220659261L;
    private CompositeType rowType;
    private List<String> indexNames;
    private transient Integer myHashCode;
    private transient String myToString;
    
    public TabularType(String s, String s2, CompositeType rowType, String[] array) throws OpenDataException {
        super(TabularData.class.getName(), s, s2, false);
        this.myHashCode = null;
        this.myToString = null;
        if (rowType == null) {
            throw new IllegalArgumentException("Argument rowType cannot be null.");
        }
        checkForNullElement(array, "indexNames");
        checkForEmptyString(array, "indexNames");
        int i = 0;
        while (i < array.length) {
            if (!rowType.containsKey(array[i])) {
                throw new OpenDataException("Argument's element value indexNames[" + i + "]=\"" + array[i] + "\" is not a valid item name for rowType.");
            }
            ++i;
        }
        this.rowType = rowType;
        ArrayList<Object> list = new ArrayList(array.length + 1);
        for (int j = 0; j < array.length; ++j) {
            list.add(array[j]);
        }
        this.indexNames = Collections.unmodifiableList(list);
        return;
    }
    
    private static void checkForNullElement(Object[] array, String s) {
        if (array == null || array.length == 0) {
            throw new IllegalArgumentException("Argument " + s + "[] cannot be null or empty.");
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] == null) {
                throw new IllegalArgumentException("Argument's element " + s + "[" + i + "] cannot be null.");
            }
            ++i;
        }
        return;
    }
    
    private static void checkForEmptyString(String[] array, String str) {
        int i = 0;
        while (i < array.length) {
            if (array[i].trim().equals("")) {
                throw new IllegalArgumentException("Argument's element " + str + "[" + i + "] cannot be an empty string.");
            }
            ++i;
        }
    }
    
    public CompositeType getRowType() {
        return this.rowType;
    }
    
    public List<String> getIndexNames() {
        return this.indexNames;
    }
    
    @Override
    public boolean isValue(Object o) {
        return o instanceof TabularData && this.isAssignableFrom(((TabularData)o).getTabularType());
    }
    
    @Override
    boolean isAssignableFrom(OpenType<?> openType) {
        if (!(openType instanceof TabularType)) {
            return false;
        }
        TabularType tabularType = (TabularType)openType;
        return this.getTypeName().equals(tabularType.getTypeName()) && this.getIndexNames().equals(tabularType.getIndexNames()) && this.getRowType().isAssignableFrom(tabularType.getRowType());
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }
        TabularType tabularType;
        try {
            tabularType = (TabularType)o;
        }
        catch (ClassCastException ex) {
            return false;
        }
        return this.getTypeName().equals(tabularType.getTypeName()) && this.rowType.equals(tabularType.rowType) && this.indexNames.equals(tabularType.indexNames);
    }
    
    @Override
    public int hashCode() {
        if (this.myHashCode == null) {
            int i = 0 + this.getTypeName().hashCode() + this.rowType.hashCode();
            Iterator<String> iterator = this.indexNames.iterator();
            while (iterator.hasNext()) {
                i += ((String)iterator.next()).hashCode();
            }
            this.myHashCode = Integer.valueOf(i);
        }
        return (int)this.myHashCode;
    }
    
    @Override
    public String toString() {
        if (this.myToString == null) {
            StringBuilder append = new StringBuilder().append(this.getClass().getName()).append("(name=").append(this.getTypeName()).append(",rowType=").append(this.rowType.toString()).append(",indexNames=(");
            String str = "";
            for (Iterator iterator = this.indexNames.iterator(); iterator.hasNext(); str = ",") {
                append.append(str).append((String)iterator.next());
            }
            append.append("))");
            this.myToString = append.toString();
        }
        return this.myToString;
    }
}
